"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
require("express-async-errors");
const http = require("http");
const cookie_parser_1 = __importDefault(require("cookie-parser"));
const dotenv = __importStar(require("dotenv"));
const express_rate_limit_1 = __importDefault(require("express-rate-limit"));
const helmet_1 = __importDefault(require("helmet"));
const cors_1 = __importDefault(require("cors"));
const socket_io_1 = require("socket.io");
const auth_1 = require("./utils/auth");
const handleErrors_1 = require("./utils/handleErrors");
const homeData_1 = require("./routers/homeData");
const game_1 = require("./routers/game");
const refreshToken_1 = require("./routers/refreshToken");
const signUp_1 = require("./routers/signUp");
const login_1 = require("./routers/login");
const config_1 = require("./config/config");
const app = (0, express_1.default)();
const server = http.createServer(app);
const io = new socket_io_1.Server(server, {
    cors: {
        origin: config_1.config.corsOrigin,
        methods: ["GET", "POST"],
    },
});
app.use((0, cors_1.default)({
    origin: config_1.config.corsOrigin,
    credentials: true,
}));
dotenv.config({ path: ".env" });
app.use(express_1.default.json());
app.use((0, helmet_1.default)());
app.use((0, cookie_parser_1.default)());
app.use((0, express_rate_limit_1.default)({
    windowMs: 5 * 60 * 1000,
    max: 500,
    standardHeaders: true,
    legacyHeaders: false,
}));
app.use("/api/sign-up", signUp_1.signUpRouter);
app.use("/api/login", login_1.loginRouter);
app.use("/api/refreshToken", refreshToken_1.refreshTokenRouter);
app.use("/api/game", auth_1.auth, game_1.gameRouter);
app.use("/api/home", auth_1.auth, homeData_1.homeDataRouter);
app.use(handleErrors_1.handleError);
io.on("connection", (socket) => {
    socket.on("join_room", (room_id) => {
        socket.join(room_id);
        socket.to(room_id).emit("player_join");
    });
    socket.on("game-move", (data) => {
        socket.to(data.room).emit("other_user_move", data);
    });
    socket.on("game-results", (data) => {
        socket.to(data.room).emit("game-results", data);
    });
    socket.on("disconnecting", function () {
        const id = socket.id;
        socket.rooms.forEach(function (room) {
            if (room != id) {
                socket.to(room).emit("user-disconnect", { room: room });
            }
        });
    });
    socket.on("user-disconnect", function () {
        const id = socket.id;
        socket.rooms.forEach(function (room) {
            if (room != id) {
                socket.to(room).emit("user-disconnect", { room: room });
            }
        });
    });
});
server.listen(3001, "0.0.0.0", () => {
    console.log("Listening on http://localhost:3001");
});
//# sourceMappingURL=index.js.map