"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoginRecord = void 0;
const handleErrors_1 = require("../utils/handleErrors");
const db_1 = require("../utils/db/db");
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
class LoginRecord {
    constructor(obj) {
        if (!obj.user_id || obj.user_id.length !== 36) {
            throw new handleErrors_1.ValidationError("Id should have 36 characters.");
        }
        if (typeof obj.user_id !== "string") {
            throw new handleErrors_1.ValidationError("The 'id' data format is invalid!");
        }
        if (!obj.refreshToken || obj.refreshToken.length > 255) {
            throw new handleErrors_1.ValidationError("The 'refreshToken' cannot be longer than 255 characters.");
        }
        if (typeof obj.refreshToken !== "string") {
            throw new handleErrors_1.ValidationError("The 'refreshToken' data format is invalid!");
        }
        this.user_id = obj.user_id;
        this.refreshToken = obj.refreshToken;
    }
    insert() {
        return __awaiter(this, void 0, void 0, function* () {
            yield db_1.pool.execute("INSERT INTO `login`(`user_id`,`refreshToken`)VALUES(:user_id,:refreshToken)", this);
        });
    }
    static getOneByRefreshToken(refreshToken) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = (yield db_1.pool.execute("SELECT * FROM `login` WHERE `refreshToken` = :refreshToken", {
                refreshToken,
            }));
            return results.length === 0 ? null : new LoginRecord(results[0]);
        });
    }
    static deleteOneByToken(refreshToken) {
        return __awaiter(this, void 0, void 0, function* () {
            yield db_1.pool.execute("DELETE FROM `login` WHERE `refreshToken` = :refreshToken", {
                refreshToken: refreshToken,
            });
        });
    }
    static createTokens(payload) {
        const token = jsonwebtoken_1.default.sign({ id: payload }, process.env.ACCESS_TOKEN_KEY, {
            expiresIn: "10min",
        });
        const refreshToken = jsonwebtoken_1.default.sign({ id: payload }, process.env.ACCESS_REFRESH_TOKEN_KEY, { expiresIn: "48h" });
        return {
            user_id: payload,
            token,
            refreshToken,
        };
    }
}
exports.LoginRecord = LoginRecord;
//# sourceMappingURL=login.record.js.map