"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RoomRecord = void 0;
const handleErrors_1 = require("../utils/handleErrors");
const db_1 = require("../utils/db/db");
const uuid_1 = require("uuid");
const types_1 = require("../types");
class RoomRecord {
    constructor(obj) {
        var _a, _b;
        if (!obj.first_user_id || obj.first_user_id.length !== 36) {
            throw new handleErrors_1.ValidationError("The 'user id' should contain 36 characters.");
        }
        if (typeof obj.first_user_id !== "string") {
            throw new handleErrors_1.ValidationError("The 'user_id' data format is invalid!");
        }
        this.room_id = (_a = obj.room_id) !== null && _a !== void 0 ? _a : (0, uuid_1.v4)();
        this.first_user_id = obj.first_user_id;
        this.second_user_id = (_b = obj.second_user_id) !== null && _b !== void 0 ? _b : null;
        this.room_type = obj.room_type;
    }
    insert() {
        return __awaiter(this, void 0, void 0, function* () {
            yield db_1.pool.execute("INSERT INTO `room`(`room_id`,`first_user_id`," +
                " `room_type`)VALUES(:room_id,:first_user_id,:room_type)", this);
            return this.room_id;
        });
    }
    static getOneByRoom_id(room_id) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = (yield db_1.pool.execute("SELECT * FROM `room` WHERE `room_id` =" + " :room_id", {
                room_id,
            }));
            return results.length === 0 ? null : results[0];
        });
    }
    static deleteRoomByUser(first_user_id) {
        return __awaiter(this, void 0, void 0, function* () {
            yield db_1.pool.execute("DELETE FROM `room` WHERE `first_user_id` = :first_user_id", {
                first_user_id: first_user_id,
            });
        });
    }
    static joinRoom(second_user_id) {
        return __awaiter(this, void 0, void 0, function* () {
            yield db_1.pool.execute("UPDATE `room` SET `second_user_id` = :second_user_id", Object.assign(Object.assign({}, this), { second_user_id: second_user_id }));
        });
    }
    static getFreeRandomRoom() {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = (yield db_1.pool.execute("SELECT * FROM `room` WHERE `room_type` = :room_type" +
                " AND  `second_user_id` IS NULL", {
                room_type: types_1.Room.random,
            }));
            return results.length === 0 ? null : results[0];
        });
    }
    static deleteRoom(room_id) {
        return __awaiter(this, void 0, void 0, function* () {
            yield db_1.pool.execute("DELETE FROM `room` WHERE `room_id` = :room_id AND  `second_user_id` IS NULL", {
                room_id: room_id,
            });
        });
    }
}
exports.RoomRecord = RoomRecord;
//# sourceMappingURL=room.record.js.map