"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserRecord = void 0;
const handleErrors_1 = require("../utils/handleErrors");
const uuid_1 = require("uuid");
const db_1 = require("../utils/db/db");
class UserRecord {
    constructor(obj) {
        var _a;
        if (!obj.email || obj.email.length > 255) {
            throw new handleErrors_1.ValidationError("Email cannot be empty or longer than 255 characters");
        }
        if (typeof obj.email !== "string") {
            throw new handleErrors_1.ValidationError("The 'email' data format is invalid!");
        }
        if (!obj.password || obj.password.length < 8 || obj.password.length > 255) {
            throw new handleErrors_1.ValidationError("The 'password' cannot be empty, shorter than 8, and longer than 255 characters.");
        }
        if (typeof obj.password !== "string") {
            throw new handleErrors_1.ValidationError("The 'password' data format is invalid!");
        }
        if (!obj.username || obj.username.length < 5 || obj.username.length > 30) {
            throw new handleErrors_1.ValidationError("The 'username' cannot be empty, shorter than 5, and longer than 30 characters.");
        }
        if (typeof obj.username !== "string") {
            throw new handleErrors_1.ValidationError("The 'username' data format is invalid!");
        }
        this.id = (_a = obj.id) !== null && _a !== void 0 ? _a : (0, uuid_1.v4)();
        this.username = obj.username;
        this.email = obj.email;
        this.password = obj.password;
    }
    insert() {
        return __awaiter(this, void 0, void 0, function* () {
            yield db_1.pool.execute("INSERT INTO `user` (`id`,`username`, `email`, `password`)VALUES(:id,:username,:email,:password)", this);
        });
    }
    static getOneByEmail(email) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = (yield db_1.pool.execute('SELECT * FROM `user` WHERE `email` = :email', {
                email,
            }));
            return results.length === 0 ? null : new UserRecord(results[0]);
        });
    }
    static getOneByUsername(username) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = (yield db_1.pool.execute('SELECT * FROM `user` WHERE `username` = :username', {
                username,
            }));
            return results.length === 0 ? null : new UserRecord(results[0]);
        });
    }
    static getOneByUserId(user_id) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = (yield db_1.pool.execute('SELECT * FROM `user` WHERE `id` = :id', {
                id: user_id,
            }));
            return results.length === 0 ? null : new UserRecord(results[0]);
        });
    }
}
exports.UserRecord = UserRecord;
//# sourceMappingURL=user.record.js.map