"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.gameRouter = void 0;
const express_1 = require("express");
const types_1 = require("../types");
const room_record_1 = require("../records/room.record");
const handleErrors_1 = require("../utils/handleErrors");
const user_record_1 = require("../records/user.record");
const play_record_1 = require("../records/play.record");
exports.gameRouter = (0, express_1.Router)();
exports.gameRouter
    .get("/start/friend/:user_id", (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const room = yield new room_record_1.RoomRecord({
            first_user_id: req.params.user_id,
            room_type: types_1.Room.friend,
        });
        const room_id = yield room.insert();
        res.json({
            room_id: room_id,
        });
    }
    catch (err) {
        throw new handleErrors_1.ValidationError("An error occurred while starting the game.");
    }
}))
    .post("/join/friend/:user_id", (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const room = yield room_record_1.RoomRecord.getOneByRoom_id(req.body.code);
    if (room === null) {
        throw new handleErrors_1.ValidationError("Invalid code.");
    }
    if (room.second_user_id !== null) {
        throw new handleErrors_1.ValidationError("Too many game participants.");
    }
    try {
        yield room_record_1.RoomRecord.joinRoom(req.params.user_id);
        res.json({
            room_id: room.room_id,
        });
    }
    catch (err) {
        throw new handleErrors_1.ValidationError("An error occurred while joining the game.");
    }
}))
    .get("/start/random/:user_id", (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    yield room_record_1.RoomRecord.deleteRoomByUser(req.params.user_id);
    const room = yield room_record_1.RoomRecord.getFreeRandomRoom();
    if (room === null) {
        try {
            const room = yield new room_record_1.RoomRecord({
                first_user_id: req.params.user_id,
                room_type: types_1.Room.random,
            });
            const room_id = yield room.insert();
            res.json({
                room_id: room_id,
                startGame: false,
            });
        }
        catch (err) {
            throw new handleErrors_1.ValidationError("An error occurred while starting the game.");
        }
    }
    else {
        try {
            yield room_record_1.RoomRecord.joinRoom(req.params.user_id);
            res.json({
                room_id: room.room_id,
                startGame: true,
            });
        }
        catch (err) {
            throw new handleErrors_1.ValidationError("An error occurred while joining the game.");
        }
    }
}))
    .get("/get-opponent/:room_id/:user_id", (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const room = yield room_record_1.RoomRecord.getOneByRoom_id(req.params.room_id);
    if (room === null) {
        throw new handleErrors_1.ValidationError("Game error. No such game room.");
    }
    if (room.first_user_id === req.params.user_id) {
        const user = yield user_record_1.UserRecord.getOneByUserId(room.second_user_id);
        if (user === null) {
            throw new handleErrors_1.ValidationError("There is no specific user in the database.");
        }
        res.json({
            opponentName: user.username,
        });
    }
    else {
        const user = yield user_record_1.UserRecord.getOneByUserId(room.first_user_id);
        if (user === null) {
            throw new handleErrors_1.ValidationError("There is no specific user in the database.");
        }
        res.json({
            opponentName: user.username,
        });
    }
}))
    .post("/save-result", (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const { user_id, points } = req.body;
    const play = yield new play_record_1.PlayRecord({
        user_id: user_id,
        points: points,
    });
    try {
        yield play.insert();
        res.json("Results saved.");
    }
    catch (err) {
        throw new handleErrors_1.ValidationError("An error occurred while saving results in database.");
    }
}));
//# sourceMappingURL=game.js.map