"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loginRouter = void 0;
const express_1 = require("express");
const handleErrors_1 = require("../utils/handleErrors");
const user_record_1 = require("../records/user.record");
const bcrypt_functions_1 = require("../utils/bcrypt-functions");
const login_record_1 = require("../records/login.record");
exports.loginRouter = (0, express_1.Router)();
exports.loginRouter
    .post("/", (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const { email, password } = req.body;
    if (req.cookies.refreshToken !== undefined &&
        (yield login_record_1.LoginRecord.getOneByRefreshToken(req.cookies.refreshToken))) {
        throw new handleErrors_1.ValidationError("User is already logged in.");
    }
    const user = yield user_record_1.UserRecord.getOneByEmail(email);
    if (user === null) {
        throw new handleErrors_1.ValidationError("The user with this email is not registered.");
    }
    if (!(0, bcrypt_functions_1.isPasswordCorrect)(password, user.password)) {
        throw new handleErrors_1.ValidationError("Password is incorrect.");
    }
    const loginDataCreated = yield login_record_1.LoginRecord.createTokens(user.id);
    const loginRecord = new login_record_1.LoginRecord({
        user_id: user.id,
        refreshToken: loginDataCreated.refreshToken,
    });
    yield loginRecord.insert();
    res
        .cookie("refreshToken", loginDataCreated.refreshToken, {
        httpOnly: true,
        maxAge: 24 * 60 * 60 * 1000,
    })
        .json({
        user_id: user.id,
        username: user.username,
        accessToken: loginDataCreated.token,
    });
}))
    .delete("/", (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const refreshToken = req.cookies.refreshToken;
    if (!refreshToken) {
        throw new handleErrors_1.ValidationError("User cannot be logged out!");
    }
    try {
        yield login_record_1.LoginRecord.getOneByRefreshToken(refreshToken);
        yield login_record_1.LoginRecord.deleteOneByToken(refreshToken);
        res.clearCookie("refreshToken").json("The user has been logged out.");
    }
    catch (err) {
        throw new handleErrors_1.ValidationError("There was an error logging out the user!");
    }
}));
//# sourceMappingURL=login.js.map