"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.refreshTokenRouter = void 0;
const express_1 = require("express");
const login_record_1 = require("../records/login.record");
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const handleErrors_1 = require("../utils/handleErrors");
const user_record_1 = require("../records/user.record");
exports.refreshTokenRouter = (0, express_1.Router)().get("/", (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const refreshToken = req.cookies.refreshToken;
    if (!refreshToken) {
        res.json("");
    }
    else {
        const result = yield login_record_1.LoginRecord.getOneByRefreshToken(refreshToken);
        if (!result) {
            res.clearCookie("refreshToken");
            throw new handleErrors_1.ValidationError("No login information in the database!");
        }
        const user = yield user_record_1.UserRecord.getOneByUserId(result.user_id);
        jsonwebtoken_1.default.verify(refreshToken, process.env.ACCESS_REFRESH_TOKEN_KEY, (err) => {
            if (err) {
                res.clearCookie("refreshToken");
                throw new handleErrors_1.ValidationError("RefreshToken verification incorrect!");
            }
            try {
                const accessToken = jsonwebtoken_1.default.sign({ user_id: result.user_id }, process.env.ACCESS_TOKEN_KEY, { expiresIn: "10min" });
                res.json({
                    accessToken: accessToken,
                    user_id: user.id,
                    username: user.username,
                });
            }
            catch (err) {
                throw new handleErrors_1.ValidationError("An error occurred while refreshing the access token.");
            }
        });
    }
}));
//# sourceMappingURL=refreshToken.js.map