"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.signUpRouter = void 0;
const express_1 = require("express");
const handleErrors_1 = require("../utils/handleErrors");
const user_record_1 = require("../records/user.record");
const bcrypt_functions_1 = require("../utils/bcrypt-functions");
exports.signUpRouter = (0, express_1.Router)();
exports.signUpRouter.post("/", (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const { username, password, confirmPassword, email, confirmEmail } = req.body;
    if (password !== confirmPassword) {
        throw new handleErrors_1.ValidationError("Passwords are incorrect.");
    }
    if (email !== confirmEmail) {
        throw new handleErrors_1.ValidationError("Emails are incorrect.");
    }
    if ((yield user_record_1.UserRecord.getOneByEmail(email)) !== null) {
        throw new handleErrors_1.ValidationError("The user with this email is already registered.");
    }
    if ((yield user_record_1.UserRecord.getOneByUsername(username)) !== null) {
        throw new handleErrors_1.ValidationError("The user with this username is already registered.");
    }
    const User = yield new user_record_1.UserRecord({
        username: username,
        password: (0, bcrypt_functions_1.hashPassword)(password),
        email: email,
    });
    try {
        yield User.insert();
    }
    catch (er) {
        throw new handleErrors_1.ValidationError("An error occurred while registering a new user.");
    }
    res.json("User registered.");
}));
//# sourceMappingURL=signUp.js.map